import chalk from 'chalk';
import ProjectNameGenerator from 'generator-jhipster/esm/generators/project-name';
import {
  PRIORITY_PREFIX,
  INITIALIZING_PRIORITY,
  PROMPTING_PRIORITY,
  CONFIGURING_PRIORITY,
  COMPOSING_PRIORITY,
  LOADING_PRIORITY,
  PREPARING_PRIORITY,
  DEFAULT_PRIORITY,
  WRITING_PRIORITY,
  POST_WRITING_PRIORITY,
  INSTALL_PRIORITY,
  END_PRIORITY,
} from 'generator-jhipster/esm/priorities';
import lodash from 'lodash';

const { startCase } = lodash;

export default class extends ProjectNameGenerator {
  constructor(args, opts, features) {
    super(args, opts, { taskPrefix: PRIORITY_PREFIX, ...features });

    if (this.options.help) return;

    if (!this.options.jhipsterContext) {
      throw new Error(`This is a JHipster blueprint and should be used only like ${chalk.yellow('jhipster --blueprints ionic')}`);
    }
  }

  get [INITIALIZING_PRIORITY]() {
    return {
      async initializingTemplateTask() {},
      ...super._initializing(),
    };
  }

  get [PROMPTING_PRIORITY]() {
    return {
      ...super._prompting(),

      // Replace prompts with custom questions
      async showPrompts() {
        if (this.shouldSkipPrompts()) return;
        const appYoRc = `${this.blueprintConfig.appDir}/.yo-rc.json`;
        const backendAppStorage = this.createStorage(appYoRc, 'generator-jhipster', { sorted: true });
        const backendAppConfig = backendAppStorage.getAll();
        const { baseName: backendAppBaseName = 'hipster' } = backendAppConfig || {};
        const defaultIonicName = `${backendAppBaseName}Ionic`;

        await this.prompt(
          [
            {
              name: 'baseName',
              type: 'input',
              validate: input => this._validateBaseName(input),
              message: 'What do you want to name your Ionic application?',
              default: defaultIonicName,
            },
            {
              name: 'projectName',
              type: 'input',
              message: 'What do you want to title your Ionic application?',
              default: ({ baseName }) => `${baseName ? startCase(baseName) : startCase(defaultIonicName)} Application`,
            },
          ],
          this.config
        );
      },
    };
  }

  get [CONFIGURING_PRIORITY]() {
    return {
      async configuringTemplateTask() {},
      ...super._configuring(),
    };
  }

  get [COMPOSING_PRIORITY]() {
    return {
      async composingTemplateTask() {},
      ...super._composing(),
    };
  }

  get [LOADING_PRIORITY]() {
    return {
      async loadingTemplateTask() {},
      ...super._loading(),
    };
  }

  get [PREPARING_PRIORITY]() {
    return {
      async preparingTemplateTask() {},
      ...super._preparing(),
    };
  }

  get [DEFAULT_PRIORITY]() {
    return {
      async defaultTemplateTask() {},
      ...super._default(),
    };
  }

  get [WRITING_PRIORITY]() {
    return {
      ...super._writing(),
    };
  }

  get [POST_WRITING_PRIORITY]() {
    return {
      async postWritingTemplateTask() {},
      ...super._postWriting(),
    };
  }

  get [INSTALL_PRIORITY]() {
    return {
      async installTemplateTask() {},
      ...super._install(),
    };
  }

  get [END_PRIORITY]() {
    return {
      async endTemplateTask() {},
      ...super._end(),
    };
  }
}
